#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import sys
from itertools import repeat

from linux.base import securecat_binary_type
from linux.base import to_native

if sys.version_info[0] == 3:
    # python 3.X
    module_iter_items = "items"


    def b(str_content):
        return str_content.encode("latin-1")


    import io

    StringIO = io.StringIO
    BytesIO = io.BytesIO
else:
    # python 2.X
    module_iter_items = "iteritems"


    def b(str_content):
        return str_content


    import StringIO

    BytesIO = StringIO.StringIO
    StringIO = StringIO.StringIO


def iteritems(key_value):
    """Return an iterator over the (key, value) pairs of a dictionary."""
    return iter(getattr(key_value, module_iter_items)())


def json_dict_bytes_to_unicode(key_value, encoding='utf-8',
                               errors='surrogate_or_strict'):
    if isinstance(key_value, dict):
        return dict(map(json_dict_bytes_to_unicode, iteritems(key_value),
                        repeat(encoding), repeat(errors)))
    elif isinstance(key_value, tuple):
        return tuple(map(json_dict_bytes_to_unicode, key_value, repeat(encoding),
                         repeat(errors)))
    elif isinstance(key_value, list):
        return list(map(json_dict_bytes_to_unicode, key_value, repeat(encoding),
                        repeat(errors)))
    elif isinstance(key_value, securecat_binary_type):
        return to_native(key_value, encoding=encoding, errors=errors)
    else:
        return key_value


class ModuleBase(object):
    def __init__(self, argument_spec):

        self.argument_spec = argument_spec
        self.params = None

    def _format_jsonify(self, json_data):
        for json_encoding in ("utf-8", "latin-1"):
            try:
                return json.dumps(json_data, encoding=json_encoding)
                # Old systems using old simplejson module does not support encoding keyword.
            except TypeError:
                try:
                    convert_data = json_dict_bytes_to_unicode(json_data,
                                                              encoding=json_encoding)
                except UnicodeDecodeError:
                    continue
                return json.dumps(convert_data)
            except UnicodeDecodeError:
                continue
        return json.dumps({"msg": "Invalid unicode encoding encountered"})

    def _return_formatted(self, kwargs):
        if 'invocation' not in kwargs:
            kwargs['invocation'] = {'module_args': self.params}
        kwargs = self._format_jsonify(kwargs)

        return kwargs

    def load_params(self, arguments):
        """load params"""
        self.params = arguments
        missing = []
        for (arguments_key, arguments_value) in self.argument_spec.items():
            # ensure all required arguments are present
            required = arguments_value.get('required', False)
            # ensure all arguments has default value
            default = arguments_value.get('default', None)
            if arguments_key not in self.params:
                if required:
                    missing.append(arguments_key)
                self.params[arguments_key] = default

        if len(missing) > 0:
            return -1, "missing required arguments: %s" % ",".join(missing)

        return 0, ''

    def exit_json(self, **kwargs):
        """ return from the module, without error """
        if not 'changed' in kwargs:
            kwargs['changed'] = False

        return self._return_formatted(kwargs)

    def fail_json(self, **kwargs):
        """ return from the module, with an error message """
        if 'msg' not in kwargs:
            return json.dumps({"msg": "implementation error -- msg to explain "
                                      "the error is required"})

        kwargs['failed'] = True

        return self._return_formatted(kwargs)
